@@ -200,6 +200,8 @@ def main(args):
     except:
         DBUS_OK = False
 
+    if args.filepath:
+        args.filepath = args.filepath.decode(sys.getfilesystemencoding()).encode(cons.STR_UTF8, cons.STR_IGNORE)
     if args.export_to_html_dir:
         lang_str = initializations()
         CherryTreeHandler(args, lang_str)
