@@ -20,19 +20,29 @@
 #       MA 02110-1301, USA.
 
 import gtk, pango, locale, os, webbrowser, re
-import cons, config
+import cons, config, exports
 
 
 def auto_decode_str(in_str, from_clipboard=False):
     """Try to Detect Encoding and Decode"""
+    try:
+        import chardet
+        enc_dict = chardet.detect(in_str)
+        encoding = enc_dict["encoding"]
+        encodings = [encoding]
+    except:
+        encodings = []
     if in_str.startswith("\xEF\xBB\xBF"): # UTF-8 "BOM"
-        encodings = ["utf-8-sig"]
+        encodings += ["utf-8-sig"]
     elif in_str.startswith(("\xFF\xFE", "\xFE\xFF")): # UTF-16 BOMs
-        encodings = [cons.STR_UTF16]
+        encodings += [cons.STR_UTF16]
     elif from_clipboard:
-        encodings = [cons.STR_UTF8, cons.STR_UTF16]
+        if "html" in in_str or "HTML" in in_str:
+            encodings = [cons.STR_UTF8] + encodings
+        else:
+            encodings = [cons.STR_UTF16, cons.STR_UTF8] + encodings
     else:
-        encodings = [cons.STR_UTF8, cons.STR_ISO_8859, cons.STR_UTF16, "utf-16le", locale.getdefaultlocale()[1]]
+        encodings += [cons.STR_UTF16, "utf-16le", cons.STR_UTF8, cons.STR_ISO_8859, locale.getdefaultlocale()[1]]
     for enc in encodings:
         try:
             out_str = in_str.decode(enc)
@@ -43,6 +53,68 @@ def auto_decode_str(in_str, from_clipboard=False):
         out_str = unicode(in_str, cons.STR_UTF8, cons.STR_IGNORE)
     return out_str
 
+def apply_tag_try_node_name(dad, iter_start, iter_end):
+    node_name = dad.curr_buffer.get_text(iter_start, iter_end)
+    node_dest = dad.get_tree_iter_from_node_name(node_name)
+    if node_dest:
+        dad.curr_buffer.select_range(iter_start, iter_end)
+        property_value = cons.LINK_TYPE_NODE + cons.CHAR_SPACE + str(dad.treestore[node_dest][3])
+        dad.apply_tag(cons.TAG_LINK, property_value=property_value)
+        return True
+    return False
+
+def apply_tag_try_link(dad, iter_end, offset_cursor=None):
+    """Try and apply link to previous word (after space or newline)"""
+    tag_applied = False
+    iter_start = iter_end.copy()
+    if iter_start.backward_char() and iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE\
+    and iter_start.backward_char() and iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE:
+        curr_state = 0
+        end_offset = iter_start.get_offset()
+        while iter_start.backward_char():
+            curr_char = iter_start.get_char()
+            if curr_char == cons.CHAR_NEWLINE:
+                break
+            if curr_char == cons.CHAR_SQ_BR_OPEN:
+                if curr_state == 0:
+                    curr_state = 1
+                else:
+                    curr_state = 2
+                    break
+        if curr_state == 2:
+            start_offset = iter_start.get_offset()+2
+            end_offset = iter_end.get_offset()-2
+            if apply_tag_try_node_name(dad, dad.curr_buffer.get_iter_at_offset(start_offset),
+                                       dad.curr_buffer.get_iter_at_offset(end_offset)):
+                tag_applied = True
+                dad.curr_buffer.delete(dad.curr_buffer.get_iter_at_offset(end_offset),
+                                       dad.curr_buffer.get_iter_at_offset(end_offset+2))
+                dad.curr_buffer.delete(dad.curr_buffer.get_iter_at_offset(start_offset-2),
+                                       dad.curr_buffer.get_iter_at_offset(start_offset))
+                if offset_cursor != None:
+                    offset_cursor -= 4
+    else:
+        iter_start = iter_end.copy()
+        while iter_start.backward_char():
+            curr_char = iter_start.get_char()
+            if curr_char in cons.WEB_LINK_SEPARATORS:
+                iter_start.forward_char()
+                break
+        num_chars = iter_end.get_offset() - iter_start.get_offset()
+        if num_chars > 4 and get_next_chars_from_iter_are(iter_start, cons.WEB_LINK_STARTERS):
+            dad.curr_buffer.select_range(iter_start, iter_end)
+            link_url = dad.curr_buffer.get_text(iter_start, iter_end)
+            if link_url[0:3] not in ["htt", "ftp"]: link_url = "http://" + link_url
+            property_value = cons.LINK_TYPE_WEBS + cons.CHAR_SPACE + link_url
+            dad.apply_tag(cons.TAG_LINK, property_value=property_value)
+            tag_applied = True
+        elif num_chars > 2 and get_is_camel_case(iter_start, num_chars):
+            if apply_tag_try_node_name(dad, iter_start, iter_end):
+                tag_applied = True
+    if tag_applied and offset_cursor != None:
+        dad.curr_buffer.place_cursor(dad.curr_buffer.get_iter_at_offset(offset_cursor))
+    return tag_applied
+
 def apply_tag_try_automatic_bounds(dad, text_buffer=None, iter_start=None):
     """Try to Select a Word Forward/Backward the Cursor"""
     if not text_buffer: text_buffer = dad.curr_buffer
@@ -109,7 +181,8 @@ def on_sourceview_event_after_button_press(dad, text_view, event):
                 dad.link_clicked(tag_name[5:], event.button == 2)
                 return False
         if dad.lists_handler.is_list_todo_beginning(text_iter):
-            dad.lists_handler.todo_list_rotate_status(text_iter, text_buffer)
+            if dad.is_curr_node_not_read_only_or_error():
+                dad.lists_handler.todo_list_rotate_status(text_iter, text_buffer)
     elif event.button == 3 and not text_buffer.get_has_selection():
         x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
         text_iter = text_view.get_iter_at_location(x, y)
@@ -118,6 +191,7 @@ def on_sourceview_event_after_button_press(dad, text_view, event):
 
 def on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, level_increase):
     """Called at list indent/unindent time"""
+    if not dad.is_curr_node_not_read_only_or_error(): return
     dad.user_active = False
     end_offset = dad.lists_handler.get_multiline_list_element_end_offset(iter_insert, list_info)
     curr_offset = list_info["startoffs"]
@@ -133,9 +207,9 @@ def on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, le
                 bull_idx = prev_list_info["num"]*(-1) - 1
             else:
                 idx_old = list_info["num"]*(-1) - 1
-                idx_offset = idx_old - curr_level % cons.NUM_CHARS_LISTBUL
-                bull_idx = (next_level + idx_offset) % cons.NUM_CHARS_LISTBUL
-            dad.replace_text_at_offset(cons.CHARS_LISTBUL[bull_idx],
+                idx_offset = idx_old - curr_level % len(dad.chars_listbul)
+                bull_idx = (next_level + idx_offset) % len(dad.chars_listbul)
+            dad.replace_text_at_offset(dad.chars_listbul[bull_idx],
                 bull_offset, bull_offset+1, text_buffer)
         else:
             idx = list_info["aux"]
@@ -167,11 +241,59 @@ def on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, le
     dad.user_active = True
     dad.update_window_save_needed("nbuf", True)
 
-def on_sourceview_event_after_key_press(dad, text_view, event):
+def on_sourceview_event_after_scroll(dad, text_view, event):
+    """Called after every gtk.gdk.SCROLL on the SourceView"""
+    if dad.ctrl_down:
+        if event.direction == gtk.gdk.SCROLL_UP:
+            dad.zoom_text_p()
+        elif event.direction == gtk.gdk.SCROLL_DOWN:
+            dad.zoom_text_m()
+    return False
+
+def on_sourceview_event_after_key_release(dad, text_view, event):
+    """Called after every gtk.gdk.KEY_RELEASE on the SourceView"""
+    if dad.ctrl_down:
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        if keyname in cons.STR_KEYS_CONTROL:
+            dad.ctrl_down = False
+    return False
+
+def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
     """Called after every gtk.gdk.KEY_PRESS on the SourceView"""
     text_buffer = text_view.get_buffer()
     keyname = gtk.gdk.keyval_name(event.keyval)
-    if (event.state & gtk.gdk.SHIFT_MASK):
+    if not dad.ctrl_down:
+        if keyname in cons.STR_KEYS_CONTROL:
+            dad.ctrl_down = True
+    if dad.auto_smart_quotes and keyname in [cons.STR_KEY_DQUOTE, cons.STR_KEY_SQUOTE]:
+        if syntax_highl in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]:
+            iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+            if iter_insert:
+                offset_1 = iter_insert.get_offset()-1
+                if offset_1 > 0:
+                    if keyname == cons.STR_KEY_DQUOTE:
+                        start_char = cons.CHAR_DQUOTE
+                        char_0 = cons.CHAR_SMART_DQUOTE_0
+                        char_1 = cons.CHAR_SMART_DQUOTE_1
+                    else:
+                        start_char = cons.CHAR_SQUOTE
+                        char_0 = cons.CHAR_SMART_SQUOTE_0
+                        char_1 = cons.CHAR_SMART_SQUOTE_1
+                    iter_start = text_buffer.get_iter_at_offset(offset_1-1)
+                    offset_0 = -1
+                    while iter_start:
+                        curr_char = iter_start.get_char()
+                        if curr_char == start_char:
+                            candidate_offset = iter_start.get_offset()
+                            if not iter_start.backward_char() or iter_start.get_char() in [cons.CHAR_NEWLINE, cons.CHAR_SPACE, cons.CHAR_TAB]:
+                                offset_0 = candidate_offset
+                            break
+                        if curr_char == cons.CHAR_NEWLINE: break
+                        if not iter_start.backward_char(): break
+                    if offset_0 >= 0:
+                        dad.replace_text_at_offset(char_0, offset_0, offset_0+1, text_buffer)
+                        dad.replace_text_at_offset(char_1, offset_1, offset_1+1, text_buffer)
+    elif (event.state & gtk.gdk.SHIFT_MASK):
         if keyname == cons.STR_KEY_RETURN:
             iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
             if not iter_insert: return False
@@ -183,6 +305,11 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
     elif keyname in [cons.STR_KEY_RETURN, cons.STR_KEY_SPACE]:
         iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         if not iter_insert: return False
+        if syntax_highl == cons.RICH_TEXT_ID:
+            iter_end_link = iter_insert.copy()
+            iter_end_link.backward_char()
+            if apply_tag_try_link(dad, iter_end_link, iter_insert.get_offset()):
+                iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         iter_start = iter_insert.copy()
         if keyname == cons.STR_KEY_RETURN:
             cursor_key_press = iter_insert.get_offset()
@@ -219,7 +346,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
             pre_spaces = 3*curr_level*cons.CHAR_SPACE if curr_level else ""
             if list_info["num"] < 0:
                 index = list_info["num"]*(-1) - 1
-                text_buffer.insert(iter_insert, pre_spaces+cons.CHARS_LISTBUL[index]+cons.CHAR_SPACE)
+                text_buffer.insert(iter_insert, pre_spaces+dad.chars_listbul[index]+cons.CHAR_SPACE)
             elif list_info["num"] == 0:
                 text_buffer.insert(iter_insert, pre_spaces+cons.CHAR_LISTTODO+cons.CHAR_SPACE)
             else:
@@ -249,13 +376,13 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
                         # at line start
                         if iter_start.get_char() == cons.CHAR_LESSER:
                             # "<> " becoming "◇ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[1], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[1], iter_start, iter_insert, text_buffer)
                         elif iter_start.get_char() == cons.CHAR_MINUS:
                             # "-> " becoming "→ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[4], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[4], iter_start, iter_insert, text_buffer)
                         elif iter_start.get_char() == cons.CHAR_EQUAL:
                             # "=> " becoming "⇒ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[5], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[5], iter_start, iter_insert, text_buffer)
                     elif iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char():
                         if iter_start.get_char() == cons.CHAR_LESSER:
                             # "<-> " becoming "↔ "
@@ -296,7 +423,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
                         dad.special_char_replace(cons.SPECIAL_CHAR_UNREGISTERED_TRADEMARK, iter_start, iter_insert, text_buffer)
                 elif iter_start.get_char() == cons.CHAR_STAR and iter_start.get_line_offset() == 0:
                     # "* " becoming "• " at line start
-                    dad.special_char_replace(cons.CHARS_LISTBUL[0], iter_start, iter_insert, text_buffer)
+                    dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[0], iter_start, iter_insert, text_buffer)
                 elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char():
                     if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN:
                         # "[] " becoming "☐ " at line start
@@ -304,7 +431,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
                 elif iter_start.get_char() == cons.CHAR_COLON and iter_start.backward_char():
                     if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_COLON:
                         # ":: " becoming "▪ " at line start
-                        dad.special_char_replace(cons.CHARS_LISTBUL[2], iter_start, iter_insert, text_buffer)
+                        dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[2], iter_start, iter_insert, text_buffer)
     return False
 
 def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
@@ -340,6 +467,8 @@ def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
         #print "link", dad.hovering_link_iter_offset
     if dad.hovering_link_iter_offset >= 0:
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
+        if len(tooltip) > cons.MAX_TOOLTIP_LINK_CHARS:
+            tooltip = tooltip[:cons.MAX_TOOLTIP_LINK_CHARS] + "..."
         text_view.set_tooltip_text(tooltip)
     else:
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
@@ -444,6 +573,30 @@ def strip_trailing_spaces(text_buffer):
                 break
     return cleaned_lines
 
+def get_is_camel_case(iter_start, num_chars):
+    """Returns True if the characters compose a camel case word"""
+    text_iter = iter_start.copy()
+    curr_state = 0
+    for i in range(num_chars):
+        curr_char = text_iter.get_char()
+        alphanumeric = re.match('\w', curr_char, re.UNICODE)
+        if not alphanumeric:
+            curr_state = -1
+            break
+        if curr_state == 0:
+            if curr_char.islower():
+                curr_state = 1
+        elif curr_state == 1:
+            if curr_char.isupper():
+                curr_state = 2
+        elif curr_state == 2:
+            if curr_char.islower():
+                curr_state = 3
+        else:
+            pass
+        text_iter.forward_char()
+    return curr_state == 3
+
 def get_next_chars_from_iter_are(iter_start, chars_list):
     """Returns True if one set of the Given Chars are the first after iter"""
     for chars in chars_list:
@@ -497,6 +650,14 @@ def get_former_line_indentation(iter_start):
         return num_tabs*cons.CHAR_TAB
     return ""
 
+def get_pango_weight(is_bold):
+    """Get pango weight (integer 200:900) heavy=900, normal=400"""
+    return pango.WEIGHT_HEAVY if is_bold else pango.WEIGHT_NORMAL
+
+def get_pango_is_bold(weight):
+    """Get True if pango weight is bold (heavy)"""
+    return weight == pango.WEIGHT_HEAVY
+
 def windows_cmd_prepare_path(filepath):
     """Prepares a Path to be digested by windows command line"""
     return cons.CHAR_DQUOTE + filepath + cons.CHAR_DQUOTE
@@ -674,12 +835,14 @@ _("Czech")+" (cs) Pavel Fric <fripohled@blogspot.com>"+cons.CHAR_NEWLINE+
 _("Dutch")+" (nl) Patrick Vijgeboom <pj.vijgeboom@gmail.com>"+cons.CHAR_NEWLINE+
 _("French")+" (fr) Klaus Becker <colonius@free.fr>"+cons.CHAR_NEWLINE+
 _("German")+" (de) Frank Brungräber <calexu@arcor.de>"+cons.CHAR_NEWLINE+
+_("Greek")+" (el) Delphina <delphina.2009@yahoo.gr>"+cons.CHAR_NEWLINE+
 _("Italian")+" (it) Vincenzo Reale <smart2128@baslug.org>"+cons.CHAR_NEWLINE+
 _("Japanese")+" (ja) Piyo <py2@live.jp>"+cons.CHAR_NEWLINE+
 _("Lithuanian")+" (lt) Zygis <zygimantus@gmail.com>"+cons.CHAR_NEWLINE+
 _("Polish")+" (pl) Marcin Swierczynski <orneo1212@gmail.com>"+cons.CHAR_NEWLINE+
 _("Portuguese Brazil")+" (pt_BR) Vinicius Schmidt <viniciussm@rocketmail.com>"+cons.CHAR_NEWLINE+
 _("Russian")+" (ru) Andriy Kovtun <kovtunos@yandex.ru>"+cons.CHAR_NEWLINE+
+_("Slovenian")+" (sl) Erik Lovrič <erik.lovric@gmail.com>"+cons.CHAR_NEWLINE+
 _("Spanish")+" (es) Daniel MC <i.e.betel@gmail.com>"+cons.CHAR_NEWLINE+
 _("Turkish")+" (tr) Ferhat Aydin <ferhataydin44@gmail.com>"+cons.CHAR_NEWLINE+
 _("Ukrainian")+" (uk) Andriy Kovtun <kovtunos@yandex.ru>")
@@ -903,8 +1066,8 @@ def dialog_image_handle(father_win, title, original_pixbuf):
     if response != gtk.RESPONSE_ACCEPT: return None
     return img_parms.original_pixbuf.scale_simple(int(img_parms.width), int(img_parms.height), gtk.gdk.INTERP_BILINEAR)
 
-def dialog_node_delete(father_win, warning_label):
-    """Confirmation before Node Remove"""
+def dialog_question_warning(father_win, warning_label):
+    """Question with Warning"""
     dialog = gtk.Dialog(title=_("Warning"),
         parent=father_win,
         flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
@@ -1406,100 +1569,122 @@ def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_
     if response != gtk.RESPONSE_ACCEPT: ret_val = 0
     return ret_val
 
-def set_bookmarks_menu_items(inst):
+def dialog_color_pick(dad, curr_color=None):
+    """Dialog to select a color, featuring a palette"""
+    dialog = gtk.ColorSelectionDialog(_("Pick a Color"))
+    dialog.set_transient_for(dad.window)
+    dialog.set_property("modal", True)
+    dialog.set_property("destroy-with-parent", True)
+    dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+    gtk_settings = gtk.settings_get_default()
+    gtk_settings.set_property("gtk-color-palette", ":".join(dad.palette_list))
+    colorselection = dialog.get_color_selection()
+    colorselection.set_has_palette(True)
+    if curr_color:
+        colorselection.set_current_color(curr_color)
+    response = dialog.run()
+    dialog.hide()
+    if response != gtk.RESPONSE_OK: return None
+    ret_color = colorselection.get_current_color()
+    color_str_hex8 = "#" + exports.rgb_any_to_24(ret_color.to_string()[1:])
+    if color_str_hex8 in dad.palette_list:
+        dad.palette_list.remove(color_str_hex8)
+    else:
+        dad.palette_list.pop()
+    dad.palette_list.insert(0, color_str_hex8)
+    return ret_color
+
+def set_bookmarks_menu_items(dad):
     """Set Bookmarks Menu Items"""
-    bookmarks_menu = inst.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
-    for menu_item in inst.bookmarks_menu_items:
+    bookmarks_menu = dad.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
+    for menu_item in dad.bookmarks_menu_items:
         bookmarks_menu.remove(menu_item)
     menu_item = gtk.SeparatorMenuItem()
     menu_item.show()
     bookmarks_menu.append(menu_item)
-    inst.bookmarks_menu_items = [menu_item]
+    dad.bookmarks_menu_items = [menu_item]
     bookmarks_to_rm = []
-    for node_id_str in inst.bookmarks:
-        if not long(node_id_str) in inst.nodes_names_dict:
+    for node_id_str in dad.bookmarks:
+        if not long(node_id_str) in dad.nodes_names_dict:
             bookmarks_to_rm.append(node_id_str)
             continue
-        menu_item = gtk.ImageMenuItem(inst.nodes_names_dict[long(node_id_str)])
+        menu_item = gtk.ImageMenuItem(dad.nodes_names_dict[long(node_id_str)])
         menu_item.set_image(gtk.image_new_from_stock("pin", gtk.ICON_SIZE_MENU))
-        menu_item.connect("activate", select_bookmark_node, node_id_str, inst)
+        menu_item.connect("activate", select_bookmark_node, node_id_str, dad)
         menu_item.show()
         bookmarks_menu.append(menu_item)
-        inst.bookmarks_menu_items.append(menu_item)
-    for element in bookmarks_to_rm: inst.bookmarks.remove(element)
+        dad.bookmarks_menu_items.append(menu_item)
+    for element in bookmarks_to_rm: dad.bookmarks.remove(element)
 
-def set_menu_items_special_chars(inst):
+def set_menu_items_special_chars(dad):
     """Set Special Chars menu items"""
-    if not "special_menu_1" in dir(inst):
-        inst.special_menu_1 = gtk.Menu()
+    if not "special_menu_1" in dir(dad):
+        dad.special_menu_1 = gtk.Menu()
         first_run = True
     else:
-        children_1 = inst.special_menu_1.get_children()
+        children_1 = dad.special_menu_1.get_children()
         for children in children_1:
-            children.hide()
-            del children
+            children.destroy()
         first_run = False
-    for special_char in inst.special_chars:
+    for special_char in dad.special_chars:
         menu_item = gtk.MenuItem(special_char)
-        menu_item.connect("activate", insert_special_char, special_char, inst)
+        menu_item.connect("activate", insert_special_char, special_char, dad)
         menu_item.show()
-        inst.special_menu_1.append(menu_item)
+        dad.special_menu_1.append(menu_item)
     if first_run:
         # main menu
         special_menuitem = gtk.ImageMenuItem(_("Insert _Special Character"))
         special_menuitem.set_image(gtk.image_new_from_stock("insert", gtk.ICON_SIZE_MENU))
         special_menuitem.set_tooltip_text(_("Insert a Special Character"))
-        special_menuitem.set_submenu(inst.special_menu_1)
-        inst.ui.get_widget("/MenuBar/EditMenu").get_submenu().insert(special_menuitem, 14)
+        special_menuitem.set_submenu(dad.special_menu_1)
+        dad.ui.get_widget("/MenuBar/EditMenu").get_submenu().insert(special_menuitem, 14)
 
-def set_menu_items_recent_documents(inst):
+def set_menu_items_recent_documents(dad):
     """Set Recent Documents menu items on Menu and Toolbar"""
-    if not "recent_menu_1" in dir(inst):
-        inst.recent_menu_1 = gtk.Menu()
-        inst.recent_menu_2 = gtk.Menu()
+    if not "recent_menu_1" in dir(dad):
+        dad.recent_menu_1 = gtk.Menu()
+        dad.recent_menu_2 = gtk.Menu()
         first_run = True
     else:
-        children_1 = inst.recent_menu_1.get_children()
-        children_2 = inst.recent_menu_2.get_children()
+        children_1 = dad.recent_menu_1.get_children()
+        children_2 = dad.recent_menu_2.get_children()
         for children in children_1:
-            children.hide()
-            del children
+            children.destroy()
         for children in children_2:
-            children.hide()
-            del children
+            children.destroy()
         first_run = False
     for target in [1, 2]:
-        for i, filepath in enumerate(inst.recent_docs):
+        for i, filepath in enumerate(dad.recent_docs):
             if i >= cons.MAX_RECENT_DOCS: break
             menu_item = gtk.ImageMenuItem(filepath)
             menu_item.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
-            menu_item.connect("activate", open_recent_document, filepath, inst)
+            menu_item.connect("activate", open_recent_document, filepath, dad)
             menu_item.show()
-            if target == 1: inst.recent_menu_1.append(menu_item)
-            else: inst.recent_menu_2.append(menu_item)
+            if target == 1: dad.recent_menu_1.append(menu_item)
+            else: dad.recent_menu_2.append(menu_item)
     if first_run:
         # main menu
         recent_menuitem = gtk.ImageMenuItem(_("_Recent Documents"))
         recent_menuitem.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
         recent_menuitem.set_tooltip_text(_("Open a Recent CherryTree Document"))
-        recent_menuitem.set_submenu(inst.recent_menu_1)
-        inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 3)
+        recent_menuitem.set_submenu(dad.recent_menu_1)
+        dad.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 3)
         # toolbar
-        if inst.toolbar_open_n_recent >= 0:
+        if dad.toolbar_open_n_recent >= 0:
             menu_toolbutton = gtk.MenuToolButton("gtk-open")
             menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
             menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
-            menu_toolbutton.set_menu(inst.recent_menu_2)
-            menu_toolbutton.connect("clicked", inst.file_open)
-            inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, inst.toolbar_open_n_recent)
+            menu_toolbutton.set_menu(dad.recent_menu_2)
+            menu_toolbutton.connect("clicked", dad.file_open)
+            dad.ui.get_widget("/ToolBar").insert(menu_toolbutton, dad.toolbar_open_n_recent)
 
-def add_recent_document(inst, filepath):
+def add_recent_document(dad, filepath):
     """Add a Recent Document if Needed"""
-    if filepath in inst.recent_docs and inst.recent_docs[0] == filepath:
+    if filepath in dad.recent_docs and dad.recent_docs[0] == filepath:
         return
-    if filepath in inst.recent_docs: inst.recent_docs.remove(filepath)
-    inst.recent_docs.insert(0, filepath)
-    set_menu_items_recent_documents(inst)
+    if filepath in dad.recent_docs: dad.recent_docs.remove(filepath)
+    dad.recent_docs.insert(0, filepath)
+    set_menu_items_recent_documents(dad)
 
 def insert_special_char(menu_item, special_char, dad):
     """A Special character insert was Requested"""
@@ -1645,11 +1830,11 @@ def bookmarks_handle(dad):
     dad.ctdb_handler.pending_edit_db_bookmarks()
     return True
 
-def set_object_highlight(inst, obj_highl):
+def set_object_highlight(dad, obj_highl):
     """Set the Highlight to obj_highl only"""
-    if inst.highlighted_obj:
-        inst.highlighted_obj.drag_unhighlight()
-        inst.highlighted_obj = None
+    if dad.highlighted_obj:
+        dad.highlighted_obj.drag_unhighlight()
+        dad.highlighted_obj = None
     if obj_highl:
         obj_highl.drag_highlight()
-        inst.highlighted_obj = obj_highl
+        dad.highlighted_obj = obj_highl
